function [Rank, ExFrm] = Evaluation(dataName, dataPath, featPath, frmPath, hvPath, Model)

    if ~exist(fullfile(featPath, [dataName, '.mat']), 'file')
        dataList{id}
        vidfeat = featExtraction(dataPath, dataName, frmPath, Model.Grid, Model.HOGDim);
        save(fullfile(featPath, [dataName, '.mat']), 'vidfeat'); 
    else
        load(fullfile(featPath, [dataName, '.mat']), 'vidfeat'); 
    end
    
    if ~exist(fullfile(hvPath, [dataName, '.mat']), 'file')
        VtScore = houghVoting(vidfeat, Model);
        save(fullfile(hvPath, [dataName, '.mat']), 'VtScore');
    else
        load(fullfile(hvPath, [dataName, '.mat']), 'VtScore');
    end

    [rowbk, colbk, frmbk, featDim] = size(vidfeat);
    vUnaryFeat    = getUnaryData(vidfeat);
    bLRBinaryFeat = getBinaryLRData(vidfeat);
    bTDBinaryFeat = getBinaryTDData(vidfeat);

    addpath(genpath('../../others/Adaboost'));
    [Cx, Fx, Fn, Fstumps] = strongJointClassifier(...
        vUnaryFeat', Model.UnaryClassifier, Model.T);
    uCScore  = Fx(:,1);
    [Cx, Fx, Fn, Fstumps] = strongJointClassifier(...
        bLRBinaryFeat', Model.LRBClassifier, Model.T);
    bLRScore = Fx(:,1);
    [Cx, Fx, Fn, Fstumps] = strongJointClassifier(...
        bTDBinaryFeat', Model.TDBClassifier, Model.T);
    bTDScore = Fx(:,1);
    rmpath(genpath('../../others/Adaboost'));
    
    UScore = zeros(rowbk, colbk, frmbk);
    idx = 1;
    for idr = 1 : rowbk
        for idc = 1 : colbk
            for idf = 1 : frmbk
                UScore(idr, idc, idf) = uCScore(idx)*VtScore(idr, idc, idf);
                idx = idx + 1;
            end
        end
    end
    
    LRScore = zeros(rowbk, colbk, frmbk);
    idx = 1;
    for idr = 1 : rowbk
        for idc = 1 : colbk-1
            for idf = 1 : frmbk
                LRScore(idr, idc, idf) = bLRScore(idx,1);
                idx = idx + 1;
            end
        end
    end
    
    TDScore = zeros(rowbk, colbk, frmbk);
    idx = 1;
    for idr = 1 : rowbk-1
        for idc = 1 : colbk
            for idf = 1 : frmbk
                TDScore(idr, idc, idf) = bTDScore(idx,1);
                idx = idx + 1;
            end
        end
    end
    
    Rank = zeros(rowbk, colbk, frmbk);
    
    for idf = 1 : frmbk
    
        crfUScore  = zeros(rowbk*colbk, 1);
        crfDMatrix = zeros(rowbk*colbk, rowbk*colbk);

        IdxArray = zeros(rowbk, colbk);
        idx = 1;
        for idr = 1 : rowbk
            for idc = 1 : colbk
                IdxArray(idr,idc) = idx;
                idx = idx+1;
            end
        end

        for idr = 1 : rowbk
            for idc = 1 : colbk
                idx = IdxArray(idr,idc);
                crfUScore(idx) = UScore(idr, idc, idf);
                if (idc-1>0)
                    lidx = IdxArray(idr, idc-1);
                    crfDMatrix(idx, lidx) = LRScore(idr, idc-1, idf);
                end
                if (idc+1<=colbk)
                    ridx = IdxArray(idr, idc+1);
                    crfDMatrix(idx, ridx) = LRScore(idr, idc, idf)*(-1);
                end
                if (idr-1>0)
                    topidx = IdxArray(idr-1, idc);
                    crfDMatrix(idx, topidx) = TDScore(idr-1, idc, idf);
                end
                if (idr+1<=rowbk)
                    downidx = IdxArray(idr+1, idc);
                    crfDMatrix(idx, downidx) = TDScore(idr, idc, idf)*(-1);
                end
            end
        end

        posIdx = find(crfDMatrix>0);
        negIdx = find(crfDMatrix<0);
        crfDMatrix(posIdx) =  1;
        crfDMatrix(negIdx) = -1;
        DAround = sum(crfDMatrix, 1);
        DAround = DAround';
        DCenter = sum(crfDMatrix, 2);

        RankScore = 1/(Model.alpha).*...
               (2.*crfUScore.*Model.alpha+Model.beta.*(DCenter-DAround));
           
        idx = 1;
        for idr = 1 : rowbk
            for idc = 1 : colbk
                Rank(idr,idc,idf) = RankScore(idx);
                idx = idx+1;
            end
        end
    end

    
