function gtdsGeneration(dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, Grid, dataList)

dataNum  = length(dataList);
parfor id = 1 : dataNum
    parProcess(id, dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, Grid, dataList);
end

function parProcess(id, dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, Grid, dataList)

    dataName = dataList{id};
    if ~(exist(fullfile(gtbkPath,  [dataName, '.mat']), 'file') & ...
    	 exist(fullfile(dsbkPath,  [dataName, '.mat']), 'file'))

        showDir = fullfile(showPath, dataName);
        mkdir(showDir);

        load(fullfile(gtPath, [dataName, '.mat']), 'BBOX', 'AppLabel');
        BBox = BBOX{1};
        
        [row, col, frmnum, frmdir] = vid2frms(dataPath, ...
            dataName, frmPath);
        
        frmnum = min(frmnum, length(AppLabel));

        nrow = ceil(row/Grid(1))*Grid(1);
        ncol = ceil(col/Grid(2))*Grid(2);

        gtextend = zeros(nrow, ncol, frmnum, 'uint8');
        dsExtend = ones(nrow, ncol, frmnum);

        rowMatrix = repmat([1:1:nrow]', 1, ncol);
        colMatrix = repmat([1:1:ncol], nrow, 1);
        rowMatrix = rowMatrix/nrow;
        colMatrix = colMatrix/ncol;
        PixPos = [rowMatrix(:), colMatrix(:)];

        BBNum = size(BBox, 3);
        for idf = 1 : frmnum
            distMatrix = ones(nrow, ncol, BBNum);
            for idb = 1 : BBNum
                bbox = squeeze(BBox(idf,:,idb));
                if sum(bbox)>0
                    gtfrm = zeros(row, col, 'uint8');
                    gtfrm(bbox(1):bbox(2), ...
                          bbox(3):bbox(4)) = 1;
                    egtfrm = imresize(gtfrm, [nrow, ncol], 'nearest');
                    gtextend(:,:,idf) = gtextend(:,:,idf) + egtfrm;
                    [rowIdx, colIdx, v] = find(egtfrm>0);
                    Center = [mean(rowIdx), mean(colIdx)];
                    Center(1) = Center(1)/nrow;
                    Center(2) = Center(2)/ncol;  
                    dist = pdist2(Center, PixPos);
                    dist = reshape(dist, nrow, ncol);
                    dist = dist/sqrt(2);
                    distMatrix(:,:,idb) = dist;
                end
            end
            if (BBNum>1)
                distMatrix = min(distMatrix,[],3);
            end
            dsExtend(:,:,idf) = distMatrix;
        end
    
        frmList = dir(fullfile(frmdir, '*.png'));
        for idf = 1 : frmnum
            im = imread(fullfile(frmdir, frmList(idf).name));
            im = imresize(rgb2gray(im), [nrow, ncol], 'nearest');
            im = double(im);
            showImI  = im*0.5 + im.*double(squeeze(gtextend(:,:,idf)))*0.5;
            showImII = im.*double(1-squeeze(dsExtend(:,:,idf)));
            showIm = [mat2gray(im), mat2gray(showImI), mat2gray(showImII)];
            imwrite(showIm, ...
                fullfile(showDir, frmList(idf).name)); 
        end
    
        bkRowStep = nrow/Grid(1); bkColStep = ncol/Grid(2);
        bkFrmStep = Grid(3);
        bkFrmBkNum = floor(frmnum/bkFrmStep);
        bkGT = zeros(Grid(1), Grid(2), bkFrmBkNum);
        dsGT = zeros(Grid(1), Grid(2), bkFrmBkNum);
        for idr = 1 : Grid(1)
            for idc = 1 : Grid(2)
                for idf = 1 : bkFrmBkNum
                    rowBgn = (idr-1)*bkRowStep+1;
                    rowEnd = idr*bkRowStep;
                    colBgn = (idc-1)*bkColStep+1;
                    colEnd = idc*bkColStep;
                    frmBgn = (idf-1)*bkFrmStep+1;
                    frmEnd = idf*bkFrmStep;
                    block = double((gtextend(rowBgn:rowEnd, colBgn:colEnd, ...
                                             frmBgn:frmEnd)&1));
                    bkGT(idr, idc, idf) = ...
                        sum(block(:))/bkRowStep/bkColStep/bkFrmStep;
                    bkCenter = [floor((rowBgn+rowEnd)/2), ...
                                floor((colBgn+colEnd)/2), ...
                                floor((frmBgn+frmEnd)/2)];
                    dsGT(idr, idc, idf) = ...
                        dsExtend(bkCenter(1), bkCenter(2), bkCenter(3));
                end
            end
        end

        save(fullfile(gtbkPath, [dataName, '.mat']), 'bkGT');
        save(fullfile(dsbkPath, [dataName, '.mat']), 'dsGT');    
    end