function RankScore = Evaluation(dataName, dataPath, featPath, hvPath, Model)

    if ~exist(fullfile(featPath, [dataName, '.mat']), 'file')
        hogfeat = featExtraction(dataPath, dataName, Model.Grid, Model.HOGDim);
        save(fullfile(featPath, [dataName, '.mat']), 'hogfeat'); 
    else
        load(fullfile(featPath, [dataName, '.mat']), 'hogfeat'); 
    end

    if ~exist(fullfile(hvPath, [dataName, '.mat']), 'file')
        VtScore = houghVoting(hogfeat, Model);
        save(fullfile(hvPath, [dataName, '.mat']), 'VtScore');
    else
        load(fullfile(hvPath, [dataName, '.mat']), 'VtScore');
    end

    [rowbk, colbk, featDim] = size(hogfeat);
    vUnaryFeat    = getUnaryData(hogfeat);
    bLRBinaryFeat = getBinaryLRData(hogfeat);
    bTDBinaryFeat = getBinaryTDData(hogfeat);
 
    addpath(genpath('../../others/Adaboost'));
    [Cx, Fx, Fn, Fstumps] = strongJointClassifier(...
        vUnaryFeat', Model.UnaryClassifier, Model.T);
    uCScore  = Fx(:,1);
    [Cx, Fx, Fn, Fstumps] = strongJointClassifier(...
        bLRBinaryFeat', Model.LRBClassifier, Model.T);
    bLRScore = Fx(:,1);
    [Cx, Fx, Fn, Fstumps] = strongJointClassifier(...
        bTDBinaryFeat', Model.TDBClassifier, Model.T);
    bTDScore = Fx(:,1);
    rmpath(genpath('../../others/Adaboost'));
    
    UScore = zeros(rowbk, colbk);
    idx = 1;
    for idr = 1 : rowbk
        for idc = 1 : colbk
            UScore(idr, idc) =VtScore(idr, idc)* uCScore(idx);
            idx = idx + 1;
        end
    end
    
    LRScore = zeros(rowbk, colbk);
    idx = 1;
    for idr = 1 : rowbk
        for idc = 1 : colbk-1
            LRScore(idr, idc) = bLRScore(idx,1);
            idx = idx + 1;
        end
    end
    
    TDScore = zeros(rowbk, colbk);
    idx = 1;
    for idr = 1 : rowbk-1
        for idc = 1 : colbk
            TDScore(idr, idc) = bTDScore(idx,1);
            idx = idx + 1;
        end
    end
    
    crfUScore  = zeros(rowbk*colbk, 1);
    crfDMatrix = zeros(rowbk*colbk, rowbk*colbk);
    
    IdxArray = zeros(rowbk, colbk);
    idx = 1;
    for idr = 1 : rowbk
        for idc = 1 : colbk
            IdxArray(idr,idc) = idx;
            idx = idx+1;
        end
    end

    for idr = 1 : rowbk
        for idc = 1 : colbk
            idx = IdxArray(idr,idc);
            crfUScore(idx) = UScore(idr, idc);
            if (idc-1>0)
                lidx = IdxArray(idr, idc-1);
                crfDMatrix(idx, lidx) = LRScore(idr, idc-1);
            end
            if (idc+1<=colbk)
            	ridx = IdxArray(idr, idc+1);
                crfDMatrix(idx, ridx) = LRScore(idr, idc)*(-1);
            end
            if (idr-1>0)
                topidx = IdxArray(idr-1, idc);
                crfDMatrix(idx, topidx) = TDScore(idr-1, idc);
            end
            if (idr+1<=rowbk)
                downidx = IdxArray(idr+1, idc);
                crfDMatrix(idx, downidx) = TDScore(idr, idc)*(-1);
            end
        end
    end
    
    posIdx = find(crfDMatrix>0);
    negIdx = find(crfDMatrix<0);
    crfDMatrix(posIdx) =  1;
    crfDMatrix(negIdx) = -1;
    DAround = sum(crfDMatrix, 1);
    DAround = DAround';
    DCenter = sum(crfDMatrix, 2);
    
    RankScore = 1/(Model.alpha).*...
           (2.*crfUScore.*Model.alpha+Model.beta.*(DCenter-DAround));
