% Actionness for Standford 40 data set
clear, clc, close all,

matlabpool open; % for parallel computation
% divide the image into 16 x 16 grids.
Model.Grid = [16, 16];

% dir to the image data, you may need to change the name of dir (images)
dataPath = '../data/images/'; 
gtPath   = '../data/annotation.mat';

% path of groundtruth for blocks
gtbkPath  = sprintf('../gtblocks_%dx%d/', Model.Grid(1), Model.Grid(2));
mkdir(gtbkPath);
% path of distance for blocks
dsbkPath  = sprintf('../dsblocks_%dx%d/', Model.Grid(1), Model.Grid(2));
mkdir(dsbkPath);
% path for showing the groundtruth
showPath  = sprintf('../showGT_%dx%d/',   Model.Grid(1), Model.Grid(2));
mkdir(showPath);

disp('Generate the groundtruth for the data.');
gtdsGeneration(dataPath, gtPath, gtbkPath, dsbkPath, showPath, Model.Grid);

% get data list
trainTxtPath = '../data/train.txt';
testTxtPath  = '../data/test.txt';
[dataList, trainList, testList, trainIdx, testIdx] = ...
    getSplit_stanford40(trainTxtPath, testTxtPath);

disp('Feature computation.');
Model.HOGDim = 9 * 5;
featPath = sprintf('../feats_%dx%d/', Model.Grid(1), Model.Grid(2));
mkdir(featPath);
featGeneration(dataPath, dataList, featPath, ...
               Model.Grid, Model.HOGDim);

disp('Train Classifiers for Unary and Binary Terms.');  
Model.T = [1 0 ; 0 1]';
Model.Nthresholds = 128;
classifierPath = sprintf('../classifier_%dx%d/', Model.Grid(1), Model.Grid(2));
mkdir(classifierPath );

Model.UnaryClassifierNum  = 256;
disp('Training for Unary Classifier.');
Model.UnaryClassifier = UnaryClassifierTrain(featPath, ...
    trainList, classifierPath, gtbkPath, Model);

disp('Training for Binary Classifier.');
Model.BinaryClassifierNum = 16;
[Model.LRBClassifier, Model.TDBClassifier] = BinaryClassifierTrain(...
    featPath, trainList, classifierPath, dsbkPath, Model);

disp('Hough Voting.')
Model.ClusterNum = 16;
cbPath =  sprintf('../codebook_%dx%d/', Model.Grid(1), Model.Grid(2));
mkdir(cbPath)
Model.Centers = CodebookGeneration(featPath, cbPath, trainList, ...
    Model.ClusterNum);
Model.Maps    = MapGeneration(featPath, cbPath, dsbkPath, trainList, ...
    Model.ClusterNum);

hvPath = sprintf('../houghvoting_%dx%dx%d', Model.Grid(1), Model.Grid(2), Model.ClusterNum);
mkdir(hvPath);
houghVotingGeneration(featPath, cbPath, dsbkPath, hvPath, ...
    dataList, Model);

disp('Evaluation.')

scorePath =  sprintf('../score_%dx%d/', Model.Grid(1), Model.Grid(2));
mkdir(scorePath)

Model.alpha =  1;
Model.beta  = -0.01;

[TrMeanAP, TrainAP] = ListEvaluation(trainList, dataPath, featPath, gtbkPath, hvPath, Model, scorePath);
txt = sprintf('trainData map: %f', TrMeanAP);
disp(txt);

[TeMeanAP, TestAP]  = ListEvaluation(testList, dataPath, featPath, gtbkPath, hvPath, Model, scorePath);
txt = sprintf('testData map: %f', TeMeanAP);
disp(txt);
matlabpool close;
