function gtdsGeneration(dataPath, gtPath, gtbkPath, dsbkPath, showPath, Grid)

load(gtPath, 'annotation');
dataNum  = length(annotation);

parfor id = 1 : dataNum
    parProcess(id, dataPath, gtPath, gtbkPath, dsbkPath, showPath, Grid, annotation{id});
end

function parProcess(id, dataPath, gtPath, gtbkPath, dsbkPath, showPath, Grid, annotation)

    dataName = annotation.imageName;
    if ~(exist(fullfile(gtbkPath,  [dataName, '.mat']), 'file') & ...
    	 exist(fullfile(dsbkPath,  [dataName, '.mat']), 'file'))
     
        bbox = annotation.bbox;
        
        im = imread(fullfile(dataPath, dataName));
        [row, col, ~] = size(im);
        gtmat = zeros(row, col, 'uint8');
        gtmat(bbox(1,2): bbox(1,4), ...
              bbox(1,1): bbox(1,3)) = 1;

        nrow = ceil(row/Grid(1))*Grid(1);
        ncol = ceil(col/Grid(2))*Grid(2);
        gtmat = imresize(gtmat, [nrow, ncol], 'nearest');

        [rowIdx, colIdx, v] = find(gtmat>0);
        Center = [mean(rowIdx), mean(colIdx)];
        Center(1) = Center(1)/nrow;
        Center(2) = Center(2)/ncol;

        rowMatrix = repmat([1:1:nrow]', 1, ncol);
        colMatrix = repmat([1:1:ncol], nrow, 1);
        rowMatrix = rowMatrix/nrow;
        colMatrix = colMatrix/ncol;
        PixPos = [rowMatrix(:), colMatrix(:)];

        dist = pdist2(Center, PixPos);
        dist = reshape(dist, nrow, ncol);
        dist = dist/sqrt(2);

        im = imresize(im, [nrow, ncol], 'nearest');

        if (size(im,3)==3)
            im = mat2gray(rgb2gray(im));
            showIm = im*0.5 + im.*double(gtmat)*0.5;
            showIm = [showIm, mat2gray(dist)];
        else
            im = mat2gray(im);
            showIm = im*0.5 + im.*double(gtmat)*0.5;
            showIm = [showIm, mat2gray(dist)];
        end

        bkRowStep = nrow/Grid(1); bkColStep = ncol/Grid(2);
        bkGT = zeros(Grid(1), Grid(2));
        dsGT = zeros(Grid(1), Grid(2));
        for idr = 1 : Grid(1)
            for idc = 1 : Grid(2)
                rowBgn = (idr-1)*bkRowStep+1;
                rowEnd = idr*bkRowStep;
                colBgn = (idc-1)*bkColStep+1;
                colEnd = idc*bkColStep;
                block = double(gtmat(rowBgn:rowEnd, colBgn:colEnd));
                bkGT(idr, idc) = sum(block(:))/bkRowStep/bkColStep;
                bkCenter = [floor((rowBgn+rowEnd)/2), ...
                            floor((colBgn+colEnd)/2)];
                dsGT(idr, idc) = dist(bkCenter(1), bkCenter(2));
             end
        end
        save(fullfile(gtbkPath,  [dataName, '.mat']), 'bkGT');
        save(fullfile(dsbkPath,  [dataName, '.mat']), 'dsGT');

        imwrite(showIm, fullfile(showPath, [dataName, '.png']));
    end
