function [LRBClassifier, TDBClassifier] = BinaryClassifierTrain(...
    featPath, trainList, classifierPath, dsPath, Model)

classifierName = sprintf('Binary_LR_Classifier_%d_%d.mat', ...
    Model.Nthresholds, Model.BinaryClassifierNum);

if ~exist(fullfile(classifierPath, classifierName), 'file')
  
    dataNum = length(trainList);
    if ~exist(fullfile(classifierPath, 'BinaryTrainLR.mat'), 'file')
        feat  = cell(dataNum,1);
        label = cell(dataNum,1);
        for id = 1 : dataNum
            load(fullfile(featPath, [trainList{id}, '.mat']), 'vidfeat');
            load(fullfile(dsPath, [trainList{id}, '.mat']), 'dsGT');
            frm = min(size(vidfeat, 3), size(dsGT, 3));
            vidfeat = vidfeat(:,:,1:frm,:);
            dsGT    = dsGT(:,:,1:frm,:);
            featitem  = getBinaryLRData(vidfeat);
            featlabel = getBinaryLRLabel(dsGT);
            feat{id}  = featitem;
            label{id} = featlabel;
        end
        save(fullfile(classifierPath, 'BinaryTrainLR.mat'), 'feat', 'label');
    else
        load(fullfile(classifierPath, 'BinaryTrainLR.mat'), 'feat', 'label');
    end
    
    TrainMatrix = cell2mat(feat);
    TrainLabel  = cell2mat(label);
    OneIdx = find(TrainLabel==1);
    TwoIdx = find(TrainLabel==2);
    OIdx   = find(TrainLabel==0);
    TrainLabel(OneIdx) = 1;
    TrainLabel(TwoIdx) = 2;
    TrainLabel(OIdx)   = 2;

    addpath(genpath('../../others/Adaboost'));

    LRBClassifier = jointBoosting(TrainMatrix', TrainLabel, Model.T, ...
        Model.BinaryClassifierNum, Model.Nthresholds);

    save(fullfile(classifierPath, classifierName), 'LRBClassifier');

    rmpath(genpath('../../others/Adaboost'));
    delete('tmp2.mat');
    
else
    load(fullfile(classifierPath, classifierName), 'LRBClassifier');
end

classifierName = sprintf('Binary_TD_Classifier_%d_%d.mat', ...
    Model.Nthresholds, Model.BinaryClassifierNum);

if ~exist(fullfile(classifierPath, classifierName), 'file')
  
    dataNum = length(trainList);
    if ~exist(fullfile(classifierPath, 'BinaryTrainTD.mat'), 'file')
        feat  = cell(dataNum,1);
        label = cell(dataNum,1);
        for id = 1 : dataNum
            load(fullfile(featPath, [trainList{id}, '.mat']), 'vidfeat');
            load(fullfile(dsPath, [trainList{id}, '.mat']), 'dsGT');
            frm = min(size(vidfeat, 3), size(dsGT, 3));
            vidfeat = vidfeat(:,:,1:frm,:);
            dsGT    = dsGT(:,:,1:frm,:);
            featitem  = getBinaryTDData(vidfeat);
            featlabel = getBinaryTDLabel(dsGT);
            feat{id}  = featitem;
            label{id} = featlabel;
        end
        save(fullfile(classifierPath, 'BinaryTrainTD.mat'), 'feat', 'label');
    else
        load(fullfile(classifierPath, 'BinaryTrainTD.mat'), 'feat', 'label');
    end
    
    TrainMatrix = cell2mat(feat);
    TrainLabel  = cell2mat(label);
    OneIdx = find(TrainLabel==1);
    TwoIdx = find(TrainLabel==2);
    OIdx   = find(TrainLabel==0);
    TrainLabel(OneIdx) = 1;
    TrainLabel(TwoIdx) = 2;
    TrainLabel(OIdx)   = 2;

    addpath(genpath('../../others/Adaboost'));

    TDBClassifier = jointBoosting(TrainMatrix', TrainLabel, Model.T, ...
        Model.BinaryClassifierNum, Model.Nthresholds);

    save(fullfile(classifierPath, classifierName), 'TDBClassifier');

    rmpath(genpath('../../others/Adaboost'));
    delete('tmp2.mat');
    
else
    load(fullfile(classifierPath, classifierName), 'TDBClassifier');
end

