% Actionness for UCF Sports data set
clear, clc, close all,

matlabpool open; % for parallel computation

% divide the frame into 16 x 16 grids.
% every 4 frames forms a block.
Model.Grid = [16, 16, 4];

% dir to the video data, you may need to change the name of dir (images)
dataPath = '../data/videos/'; 
gtPath   = '../data/gt/';

% path of groundtruth for blocks
gtbkPath  = sprintf('../gtblocks_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(gtbkPath);
% path of distance for blocks
dsbkPath  = sprintf('../dsblocks_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(dsbkPath);
% path for showing the groundtruth
showPath  = sprintf('../showGT_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(showPath);

frmPath   = '../frms';
mkdir(frmPath);

disp('Generate the groundtruth for the data.');
gtdsGeneration(dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, ...
    Model.Grid);

% get data list
[dataList, trainList, testList, trainIdx, testIdx] = ...
    getSplit_UcfSports(dataPath);

disp('Feature computation.');
%feature extraction
Model.HOGDim = 9 * 5;
featPath = sprintf('../feats_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(featPath);
featGeneration(dataPath, dataList, featPath, frmPath, ...
               Model.Grid, Model.HOGDim);

disp('Train Classifiers for Unary and Binary Terms.');  
Model.T = [1 0 ; 0 1]';
Model.Nthresholds = 128;
classifierPath = sprintf('../classifier_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(classifierPath );

disp('Training for Unary Classifier.');
Model.UnaryClassifierNum  = 256;
Model.UnaryClassifier = UnaryClassifierTrain(featPath, ...
    trainList, classifierPath, gtbkPath, Model);

disp('Training for Binary Classifier.');
Model.BinaryClassifierNum =  64;
[Model.LRBClassifier, Model.TDBClassifier] = BinaryClassifierTrain(...
    featPath, trainList, classifierPath, dsbkPath, Model);

disp('Hough Voting.')
Model.ClusterNum = 64;
cbPath = sprintf('../codebook_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(cbPath)
Model.Centers = CodebookGeneration(featPath, cbPath, trainList, ...
    Model.ClusterNum);
Model.Maps    = MapGeneration(featPath, cbPath, dsbkPath, trainList, ...
    Model.ClusterNum);
hvPath = sprintf('../houghvoting_%dx%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3), Model.ClusterNum);
mkdir(hvPath);
houghVotingGeneration(featPath, cbPath, dsbkPath, hvPath, ...
    dataList, Model);

disp('Evaluation.')

scorePath =  sprintf('../score_%dx%dx%d/', Model.Grid(1), Model.Grid(2), Model.Grid(3));
mkdir(scorePath)

Model.alpha =  1;
Model.beta  =  -0.01;

[TrMeanAP, TrainAP] = ListEvaluation(trainList, dataPath, featPath, gtbkPath, hvPath, frmPath, Model, scorePath);
txt = sprintf('trainData map: %f', TrMeanAP);
disp(txt);

[TeMeanAP, TestAP]  = ListEvaluation(testList, dataPath, featPath, gtbkPath, hvPath, frmPath, Model, scorePath);
txt = sprintf('testData map: %f', TeMeanAP);
disp(txt);

matlabpool close;
