function gtdsGeneration(dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, Grid)

dataList = dir(fullfile(dataPath, '*.avi'));
dataNum  = length(dataList);

parfor id = 1 : dataNum
    parProcess(id, dataList, dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, Grid);
end

function parProcess(id, dataList, dataPath, gtPath, gtbkPath, dsbkPath, frmPath, showPath, Grid)

    dataName = dataList(id).name;
    if ~(exist(fullfile(gtbkPath,  [dataName, '.mat']), 'file') & ...
    	 exist(fullfile(dsbkPath,  [dataName, '.mat']), 'file'))

        dataName = dataList(id).name;
        showDir = fullfile(showPath, dataName);
        mkdir(showDir);

        load(fullfile(gtPath, [dataName, '.mat']), 'BBox');

        [row, col, frmnum, frmdir] = vid2frms(dataPath, ...
            dataName, frmPath);
    
        gtFrmNum = size(BBox,1);
        frmnum = min(gtFrmNum, frmnum);
    
        nrow = ceil(row/Grid(1))*Grid(1);
        ncol = ceil(col/Grid(2))*Grid(2);

        gtextend = zeros(nrow, ncol, frmnum, 'uint8');
        dsExtend = zeros(nrow, ncol, frmnum);

        dist = zeros(nrow, ncol);
        rowMatrix = repmat([1:1:nrow]', 1, ncol);
        colMatrix = repmat([1:1:ncol], nrow, 1);
        rowMatrix = rowMatrix/nrow;
        colMatrix = colMatrix/ncol;
        PixPos = [rowMatrix(:), colMatrix(:)];

        BBNum = size(BBox, 3);
        for idf = 1 : frmnum
            distMatrix = ones(nrow, ncol, BBNum);
            for idb = 1 : BBNum
                if sum(BBox(idf,:,idb)) > 0
                    gtfrm = zeros(row, col, 'uint8');
                    gtfrm(BBox(idf,1,idb):BBox(idf,2,idb), ...
                          BBox(idf,3,idb):BBox(idf,4,idb)) = 1;
                    egtfrm = imresize(gtfrm, [nrow, ncol], 'nearest');
                    gtextend(:,:,idf) = gtextend(:,:,idf) + egtfrm;
                    [rowIdx, colIdx, v] = find(egtfrm>0);
                    Center = [mean(rowIdx), mean(colIdx)];
                    Center(1) = Center(1)/nrow;
                    Center(2) = Center(2)/ncol;  
                    dist = pdist2(Center, PixPos);
                    dist = reshape(dist, nrow, ncol);
                    dist = dist/sqrt(2);
                    distMatrix(:,:,idb) = dist;
                end
            end
            if (BBNum==2)
                distMatrix = min(distMatrix,[],3);
            end
            dsExtend(:,:,idf) = distMatrix;
        end
    
        frmList = dir(fullfile(frmdir, '*.png'));
        for idf = 1 : frmnum
            im = imread(fullfile(frmdir, frmList(idf).name));
            im = imresize(rgb2gray(im), [nrow, ncol], 'nearest');
            im = double(im);
            showImI  = im*0.5 + im.*double(squeeze(gtextend(:,:,idf)))*0.5;
            showImII = im.*double(1-squeeze(dsExtend(:,:,idf)));
            showIm = [mat2gray(im), mat2gray(showImI), mat2gray(showImII)];
            imwrite(showIm, ...
                fullfile(showDir, frmList(idf).name)); 
        end
    
        bkRowStep = nrow/Grid(1); bkColStep = ncol/Grid(2);
        bkFrmStep = Grid(3);
        bkFrmBkNum = floor(frmnum/bkFrmStep);
        bkGT = zeros(Grid(1), Grid(2), bkFrmBkNum);
        dsGT = zeros(Grid(1), Grid(2), bkFrmBkNum);
        for idr = 1 : Grid(1)
            for idc = 1 : Grid(2)
                for idf = 1 : bkFrmBkNum
                    rowBgn = (idr-1)*bkRowStep+1;
                    rowEnd = idr*bkRowStep;
                    colBgn = (idc-1)*bkColStep+1;
                    colEnd = idc*bkColStep;
                    frmBgn = (idf-1)*bkFrmStep+1;
                    frmEnd = idf*bkFrmStep;
                    block = double((gtextend(rowBgn:rowEnd, colBgn:colEnd, ...
                                             frmBgn:frmEnd)&1));
                    bkGT(idr, idc, idf) = ...
                        sum(block(:))/bkRowStep/bkColStep/bkFrmStep;
                    bkCenter = [floor((rowBgn+rowEnd)/2), ...
                                floor((colBgn+colEnd)/2), ...
                                floor((frmBgn+frmEnd)/2)];
                    dsGT(idr, idc, idf) = ...
                        dsExtend(bkCenter(1), bkCenter(2), bkCenter(3));
                end
            end
        end

        save(fullfile(gtbkPath, [dataName, '.mat']), 'bkGT');
        save(fullfile(dsbkPath, [dataName, '.mat']), 'dsGT');    
    end