function [a, b, th, featNdx, bestnode, b0, tmp, T] = fitStumpNodeC2(x, c, w, thresholds, T, tmp)
%
% All the heavy computations are only done on the leaves of the graph.
% Then, the parameters are propagated to the other nodes.  
% The thresholds are discretized (they can be different for each feature).

[Nfeatures, Nsamples] = size(x);
[Nclasses Nnodes] = size(T);
Nthresholds = size(thresholds, 2);
b0 = zeros(1, Nclasses);
error0 = zeros(1, Nclasses);

% this yields a much faster training, albeit with perhaps more nodes.  it limits the number
%  of classes that can share a feature at a node (and hence the complexity in node training
kMaxSharingDegree=8;

% this is for debug, should be removed
persistent errors



% First we compute the best parameters Ac, Bc, Wp, Wn for regression stumps for each
% independent class and for a set of predefined thresholds
if nargin < 6
    tmp = [];
end

if ~isempty(tmp)
    % reuse previous stump parameters. Only update the classes that were
    % updated in the previous round.
    Ac = tmp.Ac;
    Bc = tmp.Bc;
    Wp = tmp.Wp;
    Wn = tmp.Wn;
    b0 = tmp.b0;
    error0 = tmp.error0;
    update = tmp.update; % only update the classes that have changed
    clear tmp
else
    % in the first round, the parameters for each class have to be
    % computed. 
    Ac = zeros([Nfeatures Nthresholds Nclasses]);
    Bc = zeros([Nfeatures Nthresholds Nclasses]);
    Wp = zeros([Nfeatures Nthresholds Nclasses]);
    Wn = zeros([Nfeatures Nthresholds Nclasses]);
    update = 1:Nclasses;
end

disp(sprintf('Update stumps for %d classes out of %d', length(update), Nclasses));
disp('  Loop on leaves...')
disp('                      ')
for jn = update
    disp(sprintf('\b\b\b\b\b\b\b\b\b\b\b\b classe: %2.0f', jn))
   
    % class label for class jn: -1=background, 1=data in classe jn
    yJ = 2*double(c==jn)-1;
    wJ = w{jn};
    %we cannot normalize the weights per class: This is wrong => wJ = wJ / sum(wJ);
    
    % For each leave we fit two models: 1) a constant, 2) a stump
    b0(jn) = sum(yJ .* wJ)/ sum(wJ); 
    error0(jn) = sum(wJ.*(yJ - b0(jn)).^2);
    
    % stump
    parfor n = 1:Nfeatures
        xx  = double(x(n,:)); 
        thn = double(thresholds(n,:));
        
        [Ac(n,:,jn), Bc(n,:,jn), Wp(n,:,jn), Wn(n,:,jn)] = getLeaveStumpC(xx, yJ, wJ, thn);
        drawnow
    end
end
j = find(Wp == 0); Wp (j) = 1; % to avoid divided by zero
j = find(Wn == 0); Wn (j) = 1; % to avoid divided by zero


% Now, search for best node.
% temporarily store the sharing patterns for this search
NclapIter = min(Nclasses,kMaxSharingDegree);

Ss  = zeros(Nclasses,NclapIter);
SEs = zeros(NclapIter,1);
leaves = [1:Nclasses];

for clap=1:NclapIter
    
    SE_ = inf*ones(length(leaves),1);
    
    %parfor requires this awkward indexing setup
    parfor leafi=1:length(leaves)
        leaf = leaves(leafi);
        if (clap==1)
            S = zeros(Nclasses,1);
        else
            S = Ss(:,clap-1);
        end
        S(leaf) = 1;
        
        S = find(S);
        Snot = setdiff(1:Nclasses, S);
        
        Wns = sum(Wn(:,:,S), 3);
        Wps = sum(Wp(:,:,S), 3);
        bS = sum(Bc(:,:,S), 3) ./ Wns;
        aS = sum(Ac(:,:,S), 3) ./ Wps;
        
        SE = Wps.* (1-aS.^2) + Wns .* (1-bS.^2) + sum(error0(Snot));
        
        % we care only about the error reduction now, not the feature or
        % threshold...
        SE_(leafi) = min(SE(:));
    end
    
    [y,i] = min(SE_);
    i = leaves(i);
    % add the ith as a 1 to the S
    % copy the previous column and then go forward and add ith
    if (clap > 1)
        Ss(:,clap) = Ss(:,clap-1);
    end
    Ss(i,clap) = 1;
    
    SEs(clap) = y;
    leaves = setdiff(leaves,i);
end

[minSE,bestnode] = min(SEs);

S = Ss(:,bestnode);

% check if the node is in the graph yet
im = ismember(T',S','rows');
if (sum(im) == 0)
    % not found, need to add it
    T = [T S];
    bestnode = size(T,2);
else
    bestnode = find(im);
end

S = find(S);
Snot = setdiff(1:Nclasses, S);

Wns = sum(Wn(:,:,S), 3);
Wps = sum(Wp(:,:,S), 3);
bS = sum(Bc(:,:,S), 3) ./ Wns;
aS = sum(Ac(:,:,S), 3) ./ Wps;

SE = Wps.* (1-aS.^2) + Wns .* (1-bS.^2) + sum(error0(Snot));

% for this node, give best feature and best threshold
[merr, ndx] = min(SE,[],1);
[foo, thNdx] = min(merr);
featNdx = ndx(thNdx);

a  = aS(featNdx, thNdx);
b  = bS(featNdx, thNdx);
a  = a - b;
th = thresholds(featNdx, thNdx);


% return intermediate values for efficiency. Most of them can be reused in
% the next iteration.
tmp.Ac = Ac;
tmp.Bc = Bc;
tmp.Wp = Wp;
tmp.Wn = Wn;
tmp.b0 = b0;
tmp.error0 = error0;
tmp.update = find(T(:,bestnode))';


% debugging down here -- plot the error
errors = [errors ; foo];
%figure(1001); plot(errors,'b');
%drawnow;
save tmperrors errors

