function T = makeSharingGraph(n,type)

% function T = makeSharingGraph(n,'type')
%
%  Make the sharing graph for n classes (n here includes the '0' class)
% So, each column of T will be n-1 long.
%
% 'type' can be full
%  which will output
% T =
%     1     0     0     1     1     0     1
%     0     1     0     1     0     1     1
%     0     0     1     0     1     1     1
%  be careful -- this grows fast (2^(n-1)-1)

n = n-1;

I = eye(n);

switch(type)
    case 'full'
        T = generatetruthtable(n);
        T = T(2:size(T,1),:)';
        
    otherwise
        T = eye(n);
end



% this generatetruthtable function was found on the matlab central fiel
% exchange and here is the copyright notice
%Copyright (c) 2011, Paul Metcalf
%All rights reserved.
%
%Redistribution and use in source and binary forms, with or without 
%modification, are permitted provided that the following conditions are 
%met:
%
%    * Redistributions of source code must retain the above copyright 
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright 
%      notice, this list of conditions and the following disclaimer in 
%      the documentation and/or other materials provided with the distribution
%      
%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
%AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
%IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
%ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
%LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
%CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
%SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
%INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
%CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
%ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
%POSSIBILITY OF SUCH DAMAGE.

function [truthtable] = generatetruthtable(i)
    if (i < 1 || i > 24)                                    % Basic argument verification.
        disp('    The input argument is out of range.')     % Orders higher than 24 may be possible if you are running 64-bit?
        return
    end
    k = 0;  m = 1;  g = 2;                                  % Variable initialisation.  To break the program, change these!
    truthtable = zeros(2^i,i);
    while (m <= i)                                          % This is the start of the column loop.
        while (g <= 2^i)                                    % This is the start of the row loop.
            for (k = 0 : 2^(m-1)-1)                         % This is the start of an inter-row loop.
                truthtable(g+k,i-m+1) = 1;
                k = k+1;
            end
            g = g+2^m;
        end
        g = 2^m+1;
        m = m+1;
    end
